@@ -1,6 +1,6 @@
 /*********************************************************************/
 /* Copyright 2009, 2010 The University of Texas at Austin.           */
-/* Copyright 2023 The OpenBLAS Project.                              */
+/* Copyright 2023, 2025 The OpenBLAS Project.                        */
 /* All rights reserved.                                              */
 /*                                                                   */
 /* Redistribution and use in source and binary forms, with or        */
@@ -56,7 +56,34 @@ gotoblas_t TABLE_NAME = {
 
   GEMM_DEFAULT_OFFSET_A, GEMM_DEFAULT_OFFSET_B, GEMM_DEFAULT_ALIGN,
 
+#ifdef BUILD_HFLOAT16
+  0, 0, 0,
+  SHGEMM_DEFAULT_UNROLL_M, SHGEMM_DEFAULT_UNROLL_N,
+#ifdef SHGEMM_DEFAULT_UNROLL_MN
+ SHGEMM_DEFAULT_UNROLL_MN,
+#else
+ MAX(SHGEMM_DEFAULT_UNROLL_M, SHGEMM_DEFAULT_UNROLL_N),
+#endif
+  shgemm_kernel_SAPPHIRERAPIDS, shgemm_beta_SAPPHIRERAPIDS,
+#if SHGEMM_DEFAULT_UNROLL_M != SHGEMM_DEFAULT_UNROLL_N
+  shgemm_incopy_SAPPHIRERAPIDS, shgemm_itcopy_SAPPHIRERAPIDS,
+#else
+  shgemm_oncopy_SAPPHIRERAPIDS, shgemm_otcopy_SAPPHIRERAPIDS,
+#endif
+  shgemm_oncopy_SAPPHIRERAPIDS, shgemm_otcopy_SAPPHIRERAPIDS,
+  shgemv_n_SAPPHIRERAPIDS, shgemv_t_SAPPHIRERAPIDS,
+#endif
+
 #ifdef BUILD_BFLOAT16
+  0, 0, 0,
+  BGEMM_DEFAULT_UNROLL_M, BGEMM_DEFAULT_UNROLL_N,
+#ifdef BGEMM_DEFAULT_UNROLL_MN
+ BGEMM_DEFAULT_UNROLL_MN,
+#else
+ MAX(BGEMM_DEFAULT_UNROLL_M, BGEMM_DEFAULT_UNROLL_N),
+#endif
+  BGEMM_ALIGN_K,
+
   0, 0, 0,
   SBGEMM_DEFAULT_UNROLL_M, SBGEMM_DEFAULT_UNROLL_N,
 #ifdef SBGEMM_DEFAULT_UNROLL_MN
@@ -74,10 +101,18 @@ gotoblas_t TABLE_NAME = {
   isamax_k_SAPPHIRERAPIDS, isamin_k_SAPPHIRERAPIDS, ismax_k_SAPPHIRERAPIDS, ismin_k_SAPPHIRERAPIDS,
   snrm2_k_SAPPHIRERAPIDS,  sasum_k_SAPPHIRERAPIDS,  ssum_k_SAPPHIRERAPIDS, scopy_k_SAPPHIRERAPIDS, sbdot_k_SAPPHIRERAPIDS,
   dsdot_k_SAPPHIRERAPIDS,
-  srot_k_SAPPHIRERAPIDS,   srotm_k_SAPPHIRERAPIDS,  saxpy_k_SAPPHIRERAPIDS,  sscal_k_SAPPHIRERAPIDS, sswap_k_SAPPHIRERAPIDS,
-  sbgemv_n_SAPPHIRERAPIDS, sbgemv_t_SAPPHIRERAPIDS, sger_k_SAPPHIRERAPIDS,
+  srot_k_SAPPHIRERAPIDS,   srotm_k_SAPPHIRERAPIDS, bscal_k_SAPPHIRERAPIDS, saxpy_k_SAPPHIRERAPIDS,  sscal_k_SAPPHIRERAPIDS, sswap_k_SAPPHIRERAPIDS,
+  bgemv_n_SAPPHIRERAPIDS, bgemv_t_SAPPHIRERAPIDS, sbgemv_n_SAPPHIRERAPIDS, sbgemv_t_SAPPHIRERAPIDS, sger_k_SAPPHIRERAPIDS,
   ssymv_L_SAPPHIRERAPIDS, ssymv_U_SAPPHIRERAPIDS,
 
+  bgemm_kernel_SAPPHIRERAPIDS, bgemm_beta_SAPPHIRERAPIDS,
+#if BGEMM_DEFAULT_UNROLL_M != BGEMM_DEFAULT_UNROLL_N
+  bgemm_incopy_SAPPHIRERAPIDS, bgemm_itcopy_SAPPHIRERAPIDS,
+#else
+  bgemm_oncopy_SAPPHIRERAPIDS, bgemm_otcopy_SAPPHIRERAPIDS,
+#endif
+  bgemm_oncopy_SAPPHIRERAPIDS, bgemm_otcopy_SAPPHIRERAPIDS,
+
   sbgemm_kernel_SAPPHIRERAPIDS, sbgemm_beta_SAPPHIRERAPIDS,
 #if SBGEMM_DEFAULT_UNROLL_M != SBGEMM_DEFAULT_UNROLL_N
   sbgemm_incopy_SAPPHIRERAPIDS, sbgemm_itcopy_SAPPHIRERAPIDS,
@@ -181,6 +216,22 @@ gotoblas_t TABLE_NAME = {
 #endif
 #ifdef ARCH_ARM64
   sgemm_direct_SAPPHIRERAPIDS,
+  sgemm_direct_performant_SAPPHIRERAPIDS,	
+  sgemm_direct_alpha_beta_SAPPHIRERAPIDS,
+  ssymm_direct_alpha_betaLU_SAPPHIRERAPIDS,
+  ssymm_direct_alpha_betaLL_SAPPHIRERAPIDS,
+  strmm_direct_LNUN_SAPPHIRERAPIDS,
+  strmm_direct_LNLN_SAPPHIRERAPIDS,
+  strmm_direct_LTUN_SAPPHIRERAPIDS,
+  strmm_direct_LTLN_SAPPHIRERAPIDS,
+  ssyrk_direct_alpha_betaUN_SAPPHIRERAPIDS,
+  ssyrk_direct_alpha_betaUT_SAPPHIRERAPIDS,
+  ssyrk_direct_alpha_betaLN_SAPPHIRERAPIDS,
+  ssyrk_direct_alpha_betaLT_SAPPHIRERAPIDS,
+  ssyr2k_direct_alpha_betaUN_SAPPHIRERAPIDS,
+  ssyr2k_direct_alpha_betaUT_SAPPHIRERAPIDS,
+  ssyr2k_direct_alpha_betaLN_SAPPHIRERAPIDS,
+  ssyr2k_direct_alpha_betaLT_SAPPHIRERAPIDS,
 #endif
 
   sgemm_kernel_SAPPHIRERAPIDS, sgemm_beta_SAPPHIRERAPIDS,
@@ -896,6 +947,7 @@ gotoblas_t TABLE_NAME = {
 static void init_parameter(void) {
 #if (BUILD_BFLOAT16)
   TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+  TABLE_NAME.bgemm_p = BGEMM_DEFAULT_P;
 #endif
 #if (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
@@ -912,6 +964,7 @@ static void init_parameter(void) {
 
 #if (BUILD_BFLOAT16)
   TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+  TABLE_NAME.bgemm_q = BGEMM_DEFAULT_Q;
 #endif
 #if BUILD_SINGLE == 1 || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
@@ -928,6 +981,7 @@ static void init_parameter(void) {
 
 #if (BUILD_BFLOAT16)
   TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+  TABLE_NAME.bgemm_r = BGEMM_DEFAULT_R;
 #endif
 #if BUILD_SINGLE == 1 || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
@@ -1083,10 +1137,12 @@ static void init_parameter(void) {
 
 #ifdef BUILD_BFLOAT16
   TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+  TABLE_NAME.bgemm_p = BGEMM_DEFAULT_P;
 #endif
 
 #ifdef BUILD_BFLOAT16
   TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+  TABLE_NAME.bgemm_r = BGEMM_DEFAULT_R;
 #endif
 
 #if defined(LA464)
@@ -1185,6 +1241,7 @@ static void init_parameter(void) {
 
 #ifdef BUILD_BFLOAT16
   TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+  TABLE_NAME.bgemm_q = BGEMM_DEFAULT_Q;
 #endif
 }
 #else // (ARCH_LOONGARCH64)
@@ -1193,6 +1250,7 @@ static void init_parameter(void) {
 
 #ifdef BUILD_BFLOAT16
   TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+  TABLE_NAME.bgemm_p = BGEMM_DEFAULT_P;
 #endif
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
@@ -1201,6 +1259,7 @@ static void init_parameter(void) {
 
 #ifdef BUILD_BFLOAT16
   TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+  TABLE_NAME.bgemm_r = BGEMM_DEFAULT_R;
 #endif
   TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
   TABLE_NAME.dgemm_r = DGEMM_DEFAULT_R;
@@ -1210,6 +1269,7 @@ static void init_parameter(void) {
 
 #ifdef BUILD_BFLOAT16
   TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+  TABLE_NAME.bgemm_q = BGEMM_DEFAULT_Q;
 #endif
   TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
   TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
@@ -1222,6 +1282,7 @@ static void init_parameter(void) {
 static void init_parameter(void) {
 #ifdef BUILD_BFLOAT16
 	TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+	TABLE_NAME.bgemm_p = BGEMM_DEFAULT_P;
 #endif
 	TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
 	TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
@@ -1230,6 +1291,7 @@ static void init_parameter(void) {
 
 #ifdef BUILD_BFLOAT16
 	TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+	TABLE_NAME.bgemm_r = BGEMM_DEFAULT_R;
 #endif
 	TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
 	TABLE_NAME.dgemm_r = DGEMM_DEFAULT_R;
@@ -1239,6 +1301,7 @@ static void init_parameter(void) {
 
 #ifdef BUILD_BFLOAT16
 	TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+	TABLE_NAME.bgemm_q = BGEMM_DEFAULT_Q;
 #endif
 	TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
 	TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
@@ -1252,6 +1315,10 @@ static void init_parameter(void) {
 
 #ifdef BUILD_BFLOAT16
   TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
+  TABLE_NAME.bgemm_p = BGEMM_DEFAULT_P;
+#endif
+#ifdef BUILD_HFLOAT16
+  TABLE_NAME.shgemm_p = SHGEMM_DEFAULT_P;
 #endif
   TABLE_NAME.sgemm_p = SGEMM_DEFAULT_P;
   TABLE_NAME.dgemm_p = DGEMM_DEFAULT_P;
@@ -1260,6 +1327,10 @@ static void init_parameter(void) {
 
 #ifdef BUILD_BFLOAT16
   TABLE_NAME.sbgemm_r = SBGEMM_DEFAULT_R;
+  TABLE_NAME.bgemm_r = BGEMM_DEFAULT_R;
+#endif
+#ifdef BUILD_HFLOAT16
+  TABLE_NAME.shgemm_r = SHGEMM_DEFAULT_R;
 #endif
   TABLE_NAME.sgemm_r = SGEMM_DEFAULT_R;
   TABLE_NAME.dgemm_r = DGEMM_DEFAULT_R;
@@ -1269,6 +1340,10 @@ static void init_parameter(void) {
 
 #ifdef BUILD_BFLOAT16
   TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+  TABLE_NAME.bgemm_q = BGEMM_DEFAULT_Q;
+#endif
+#ifdef BUILD_HFLOAT16
+  TABLE_NAME.shgemm_q = SHGEMM_DEFAULT_Q;
 #endif
   TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
   TABLE_NAME.dgemm_q = DGEMM_DEFAULT_Q;
@@ -1416,6 +1491,12 @@ static void init_parameter(void) {
 #ifdef BUILD_BFLOAT16
   TABLE_NAME.sbgemm_p = SBGEMM_DEFAULT_P;
   TABLE_NAME.sbgemm_q = SBGEMM_DEFAULT_Q;
+  TABLE_NAME.bgemm_p = BGEMM_DEFAULT_P;
+  TABLE_NAME.bgemm_q = BGEMM_DEFAULT_Q;
+#endif
+#ifdef BUILD_HFLOAT16
+  TABLE_NAME.shgemm_p = SHGEMM_DEFAULT_P;
+  TABLE_NAME.shgemm_q = SHGEMM_DEFAULT_Q;
 #endif
 #if  (BUILD_SINGLE==1) || (BUILD_COMPLEX==1)
   TABLE_NAME.sgemm_q = SGEMM_DEFAULT_Q;
@@ -2010,6 +2091,17 @@ static void init_parameter(void) {
 			       ((TABLE_NAME.sbgemm_p * TABLE_NAME.sbgemm_q *  4 + TABLE_NAME.offsetA
 				 + TABLE_NAME.align) & ~TABLE_NAME.align)
 			       ) / (TABLE_NAME.sbgemm_q *  4) - 15) & ~15);
+  TABLE_NAME.bgemm_r = (((BUFFER_SIZE -
+			       ((TABLE_NAME.bgemm_p * TABLE_NAME.bgemm_q *  4 + TABLE_NAME.offsetA
+				 + TABLE_NAME.align) & ~TABLE_NAME.align)
+			       ) / (TABLE_NAME.bgemm_q *  4) - 15) & ~15);
+#endif
+
+#if BUILD_HFLOAT16==1
+  TABLE_NAME.shgemm_r = (((BUFFER_SIZE -
+			       ((TABLE_NAME.shgemm_p * TABLE_NAME.shgemm_q *  4 + TABLE_NAME.offsetA
+				 + TABLE_NAME.align) & ~TABLE_NAME.align)
+			       ) / (TABLE_NAME.shgemm_q *  4) - 15) & ~15);
 #endif
 
 #if BUILD_SINGLE==1
